# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/api/featureservice/feature_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.api.featureservice import feature_service_request_pb2 as tecton__proto_dot_api_dot_featureservice_dot_feature__service__request__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from protoc_gen_swagger.options import annotations_pb2 as protoc__gen__swagger_dot_options_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/api/featureservice/feature_service.proto',
  package='tecton_proto.api.feature_service',
  syntax='proto2',
  serialized_options=b'Z4github.com/tecton-ai/tecton_proto/api/featureservice\222A\226\003\022\354\002\n\022FeatureService API\022\264\002Tecton provides a low latency feature server that exposes HTTP endpoints to retrieve feature values and metadata from the online store. This is typically used during model predictions. The feature servers retrieve data from the online store and perform any additional aggregations and filtering as necessary.\"\013\n\tTecton AI*\r\n\013Proprietary2\0031.0*\001\0022\020application/json:\020application/json',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n5tecton_proto/api/featureservice/feature_service.proto\x12 tecton_proto.api.feature_service\x1a=tecton_proto/api/featureservice/feature_service_request.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a,protoc-gen-swagger/options/annotations.proto\"\xcf\x01\n\x0bNameAndType\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12[\n\tdata_type\x18\x03 \x01(\x0b\x32>.tecton_proto.api.feature_service.FeatureServerComplexDataTypeR\x08\x64\x61taType\x12O\n\x04type\x18\x02 \x01(\x0e\x32\x37.tecton_proto.api.feature_service.FeatureServerDataTypeB\x02\x18\x01R\x04type\"\x94\x01\n\x11GetFeaturesResult\x12\x32\n\x08\x66\x65\x61tures\x18\x05 \x03(\x0b\x32\x16.google.protobuf.ValueR\x08\x66\x65\x61tures\x12\x33\n\tjoin_keys\x18\x06 \x03(\x0b\x32\x16.google.protobuf.ValueR\x08joinKeysJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05\"\xce\x01\n\x1c\x46\x65\x61tureServerComplexDataType\x12K\n\x04type\x18\x01 \x01(\x0e\x32\x37.tecton_proto.api.feature_service.FeatureServerDataTypeR\x04type\x12\x61\n\x0c\x65lement_type\x18\x02 \x01(\x0b\x32>.tecton_proto.api.feature_service.FeatureServerComplexDataTypeR\x0b\x65lementType\"\xdf\x03\n\x07SloInfo\x12!\n\x0cslo_eligible\x18\x01 \x01(\x08R\x0bsloEligible\x12\x35\n\x17slo_server_time_seconds\x18\x05 \x01(\x01R\x14sloServerTimeSeconds\x12t\n\x19slo_ineligibility_reasons\x18\x02 \x03(\x0e\x32\x38.tecton_proto.api.feature_service.SloIneligibilityReasonR\x17sloIneligibilityReasons\x12?\n\x1c\x64ynamodb_response_size_bytes\x18\x03 \x01(\x05R\x19\x64ynamodbResponseSizeBytes\x12.\n\x13server_time_seconds\x18\x04 \x01(\x01R\x11serverTimeSeconds\x12,\n\x12store_time_seconds\x18\x06 \x01(\x01R\x10storeTimeSeconds\x12*\n\x11store_max_latency\x18\x07 \x01(\x01R\x0fstoreMaxLatency\x12\x39\n\x19store_response_size_bytes\x18\x08 \x01(\x05R\x16storeResponseSizeBytes\"\xa2\x04\n\x08Metadata\x12K\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32/.tecton_proto.api.feature_service.Metadata.ItemR\x08\x66\x65\x61tures\x12L\n\tjoin_keys\x18\x02 \x03(\x0b\x32/.tecton_proto.api.feature_service.Metadata.ItemR\x08joinKeys\x12\x44\n\x08slo_info\x18\x03 \x01(\x0b\x32).tecton_proto.api.feature_service.SloInfoR\x07sloInfo\x12\'\n\x0fpartial_results\x18\x04 \x01(\x08R\x0epartialResults\x1a\x8b\x02\n\x04Item\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x41\n\x0e\x65\x66\x66\x65\x63tive_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\reffectiveTime\x12O\n\x04type\x18\x03 \x01(\x0e\x32\x37.tecton_proto.api.feature_service.FeatureServerDataTypeB\x02\x18\x01R\x04type\x12[\n\tdata_type\x18\x04 \x01(\x0b\x32>.tecton_proto.api.feature_service.FeatureServerComplexDataTypeR\x08\x64\x61taType\"\xaa\x01\n\x13GetFeaturesResponse\x12K\n\x06result\x18\x02 \x01(\x0b\x32\x33.tecton_proto.api.feature_service.GetFeaturesResultR\x06result\x12\x46\n\x08metadata\x18\x03 \x01(\x0b\x32*.tecton_proto.api.feature_service.MetadataR\x08metadata\"\xaf\x01\n\x18GetFeaturesBatchResponse\x12K\n\x06result\x18\x02 \x03(\x0b\x32\x33.tecton_proto.api.feature_service.GetFeaturesResultR\x06result\x12\x46\n\x08metadata\x18\x03 \x01(\x0b\x32*.tecton_proto.api.feature_service.MetadataR\x08metadata\"i\n\x16ServiceMetadataRequest\x12O\n\x06params\x18\x01 \x01(\x0b\x32\x37.tecton_proto.api.feature_service.FeatureServiceLocatorR\x06params\"\xf3\x03\n\x17ServiceMetadataResponse\x12\x66\n\x14\x66\x65\x61ture_service_type\x18\x01 \x01(\x0e\x32\x34.tecton_proto.api.feature_service.FeatureServiceTypeR\x12\x66\x65\x61tureServiceType\x12U\n\x0finput_join_keys\x18\x02 \x03(\x0b\x32-.tecton_proto.api.feature_service.NameAndTypeR\rinputJoinKeys\x12j\n\x1ainput_request_context_keys\x18\x03 \x03(\x0b\x32-.tecton_proto.api.feature_service.NameAndTypeR\x17inputRequestContextKeys\x12W\n\x10output_join_keys\x18\x04 \x03(\x0b\x32-.tecton_proto.api.feature_service.NameAndTypeR\x0eoutputJoinKeys\x12T\n\x0e\x66\x65\x61ture_values\x18\x05 \x03(\x0b\x32-.tecton_proto.api.feature_service.NameAndTypeR\rfeatureValues\"g\n\x14QueryFeaturesRequest\x12O\n\x06params\x18\x01 \x01(\x0b\x32\x37.tecton_proto.api.feature_service.GetFeaturesParametersR\x06params\"\xae\x01\n\x15QueryFeaturesResponse\x12M\n\x07results\x18\x01 \x03(\x0b\x32\x33.tecton_proto.api.feature_service.GetFeaturesResultR\x07results\x12\x46\n\x08metadata\x18\x02 \x01(\x0b\x32*.tecton_proto.api.feature_service.MetadataR\x08metadata\"\xa2\x03\n\x15\x46\x65\x61tureServiceLocator\x12.\n\x12\x66\x65\x61ture_service_id\x18\x01 \x01(\tH\x00R\x10\x66\x65\x61tureServiceId\x12\x32\n\x14\x66\x65\x61ture_service_name\x18\x02 \x01(\tH\x00R\x12\x66\x65\x61tureServiceName\x12.\n\x12\x66\x65\x61ture_package_id\x18\x04 \x01(\tH\x00R\x10\x66\x65\x61turePackageId\x12\x32\n\x14\x66\x65\x61ture_package_name\x18\x05 \x01(\tH\x00R\x12\x66\x65\x61turePackageName\x12(\n\x0f\x66\x65\x61ture_view_id\x18\x06 \x01(\tH\x00R\rfeatureViewId\x12,\n\x11\x66\x65\x61ture_view_name\x18\x07 \x01(\tH\x00R\x0f\x66\x65\x61tureViewName\x12%\n\x0eworkspace_name\x18\x03 \x01(\tR\rworkspaceName:7\x92\x41\x34:2{\"feature_service_name\": \"ctr_prediction_service\"}B\t\n\x07locator\"\x90\x01\n\x1dGetFeatureServiceStateRequest\x12o\n\x17\x66\x65\x61ture_service_locator\x18\x01 \x01(\x0b\x32\x37.tecton_proto.api.feature_service.FeatureServiceLocatorR\x15\x66\x65\x61tureServiceLocator\"&\n\x1eGetFeatureServiceStateResponseJ\x04\x08\x01\x10\x02*/\n\x12\x46\x65\x61tureServiceType\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x0c\n\x08WILDCARD\x10\x01*\x84\x01\n\x15\x46\x65\x61tureServerDataType\x12\x10\n\x0cmissing_type\x10\x00\x12\x0b\n\x07\x62oolean\x10\x01\x12\x0b\n\x07\x66loat64\x10\x02\x12\t\n\x05int64\x10\x03\x12\n\n\x06string\x10\x04\x12\x10\n\x0cstring_array\x10\x05\x12\t\n\x05\x61rray\x10\x06\x12\x0b\n\x07\x66loat32\x10\x07*\x9a\x01\n\x16SloIneligibilityReason\x12\x0b\n\x07UNKNOWN\x10\x00\x12)\n%DYNAMODB_RESPONSE_SIZE_LIMIT_EXCEEDED\x10\x01\x12&\n\"REDIS_RESPONSE_SIZE_LIMIT_EXCEEDED\x10\x02\x12 \n\x1cREDIS_LATENCY_LIMIT_EXCEEDED\x10\x03\x32\x9a\x13\n\x0e\x46\x65\x61tureService\x12\xf3\x03\n\x0bGetFeatures\x12\x34.tecton_proto.api.feature_service.GetFeaturesRequest\x1a\x35.tecton_proto.api.feature_service.GetFeaturesResponse\"\xf6\x02\x82\xd3\xe4\x93\x02%\" /v1/feature-service/get-features:\x01*\x92\x41\xc7\x02\n\x0bGetFeatures\x1a\x46Get Features API to retrieve feature values from Tecton\'s online storeJ?\n\x03\x32\x30\x30\x12\x38\n\x02OK\"2\n\x10\x61pplication/json\x12\x1e{\"result\": {\"features\": [15]}}J\xae\x01\n\x03\x34\x30\x34\x12\xa6\x01\n\tNot Found\"\x98\x01\n\x10\x61pplication/json\x12\x83\x01{\"error\": \"Unknown feature_service_id: my_feature_service\", \"message\": \"Unknown feature_service_id: my_feature_service\", \"code\": 5}\x12\xd9\x05\n\x10GetFeaturesBatch\x12\x39.tecton_proto.api.feature_service.GetFeaturesBatchRequest\x1a:.tecton_proto.api.feature_service.GetFeaturesBatchResponse\"\xcd\x04\x82\xd3\xe4\x93\x02+\"&/v1/feature-service/get-features-batch:\x01*\x92\x41\x98\x04\n\x10GetFeaturesBatch\x1aUBatch get Features API to retrieve feature values in batch from Tecton\'s online storeJ^\n\x03\x32\x30\x30\x12W\n\x02OK\"Q\n\x10\x61pplication/json\x12={\"result\": {\"features\": [[15, 10, 5, 100],[10, 20, 1, 200]]}}J\xae\x01\n\x03\x34\x30\x34\x12\xa6\x01\n\tNot Found\"\x98\x01\n\x10\x61pplication/json\x12\x83\x01{\"error\": \"Unknown feature_service_id: my_feature_service\", \"message\": \"Unknown feature_service_id: my_feature_service\", \"code\": 5}J\x9b\x01\n\x03\x34\x30\x30\x12\x93\x01\n\x0b\x42\x61\x64 Request\"\x83\x01\n\x10\x61pplication/json\x12o{\"error\": \"Batch size 27 exceeds maximum of 25.\", \"message\": \"Batch size 27 exceeds maximum of 25.\", \"code\": 3}\x12\xaf\x01\n\rQueryFeatures\x12\x36.tecton_proto.api.feature_service.QueryFeaturesRequest\x1a\x37.tecton_proto.api.feature_service.QueryFeaturesResponse\"-\x82\xd3\xe4\x93\x02\'\"\"/v1/feature-service/query-features:\x01*\x12\xd5\x01\n\x16GetFeatureServiceState\x12?.tecton_proto.api.feature_service.GetFeatureServiceStateRequest\x1a@.tecton_proto.api.feature_service.GetFeatureServiceStateResponse\"8\x82\xd3\xe4\x93\x02\x32\"-/v1/feature-service/get-feature-service-state:\x01*\x12\xab\x06\n\x0fServiceMetadata\x12\x38.tecton_proto.api.feature_service.ServiceMetadataRequest\x1a\x39.tecton_proto.api.feature_service.ServiceMetadataResponse\"\xa2\x05\x82\xd3\xe4\x93\x02!\"\x1c/v1/feature-service/metadata:\x01*\x92\x41\xf7\x04\n\x08Metadata\x1a\x32Get Metadata about the feature service from TectonJ\xd9\x02\n\x03\x32\x30\x30\x12\xd1\x02\n\x02OK\"\xca\x02\n\x10\x61pplication/json\x12\xb5\x02{\"featureServiceType\": \"DEFAULT\", \"inputJoinKeys\": [{\"name\": \"adId\", \"type\": \"string\"}],\"feature_values\":[{\"name\": \"ad_ground_truth_ctr_performance_7_days\", \"type\": \"float64\"}, {\"name\": \"user_partner_impression_count_7_days\", \"type\": \"float64\"}, {\"name\": \"user_total_ad_frequency_counts\", \"type\": \"int64\"} ] }J\xda\x01\n\x03\x34\x30\x34\x12\xd2\x01\n\tNot Found\"\xc4\x01\n\x10\x61pplication/json\x12\xaf\x01{\"error\": \"Unknown feature_service_name: my_feature_service for workspace: prod\", \"message\": \"Unknown feature_service_name: my_feature_service for workspace: prod\", \"code\": 5}B\xd0\x03Z4github.com/tecton-ai/tecton_proto/api/featureservice\x92\x41\x96\x03\x12\xec\x02\n\x12\x46\x65\x61tureService API\x12\xb4\x02Tecton provides a low latency feature server that exposes HTTP endpoints to retrieve feature values and metadata from the online store. This is typically used during model predictions. The feature servers retrieve data from the online store and perform any additional aggregations and filtering as necessary.\"\x0b\n\tTecton AI*\r\n\x0bProprietary2\x03\x31.0*\x01\x02\x32\x10\x61pplication/json:\x10\x61pplication/json'
  ,
  dependencies=[tecton__proto_dot_api_dot_featureservice_dot_feature__service__request__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,protoc__gen__swagger_dot_options_dot_annotations__pb2.DESCRIPTOR,])

_FEATURESERVICETYPE = _descriptor.EnumDescriptor(
  name='FeatureServiceType',
  full_name='tecton_proto.api.feature_service.FeatureServiceType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DEFAULT', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='WILDCARD', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3744,
  serialized_end=3791,
)
_sym_db.RegisterEnumDescriptor(_FEATURESERVICETYPE)

FeatureServiceType = enum_type_wrapper.EnumTypeWrapper(_FEATURESERVICETYPE)
_FEATURESERVERDATATYPE = _descriptor.EnumDescriptor(
  name='FeatureServerDataType',
  full_name='tecton_proto.api.feature_service.FeatureServerDataType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='missing_type', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='boolean', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='float64', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='int64', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='string', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='string_array', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='array', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='float32', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3794,
  serialized_end=3926,
)
_sym_db.RegisterEnumDescriptor(_FEATURESERVERDATATYPE)

FeatureServerDataType = enum_type_wrapper.EnumTypeWrapper(_FEATURESERVERDATATYPE)
_SLOINELIGIBILITYREASON = _descriptor.EnumDescriptor(
  name='SloIneligibilityReason',
  full_name='tecton_proto.api.feature_service.SloIneligibilityReason',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DYNAMODB_RESPONSE_SIZE_LIMIT_EXCEEDED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REDIS_RESPONSE_SIZE_LIMIT_EXCEEDED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='REDIS_LATENCY_LIMIT_EXCEEDED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3929,
  serialized_end=4083,
)
_sym_db.RegisterEnumDescriptor(_SLOINELIGIBILITYREASON)

SloIneligibilityReason = enum_type_wrapper.EnumTypeWrapper(_SLOINELIGIBILITYREASON)
DEFAULT = 0
WILDCARD = 1
missing_type = 0
boolean = 1
float64 = 2
int64 = 3
string = 4
string_array = 5
array = 6
float32 = 7
UNKNOWN = 0
DYNAMODB_RESPONSE_SIZE_LIMIT_EXCEEDED = 1
REDIS_RESPONSE_SIZE_LIMIT_EXCEEDED = 2
REDIS_LATENCY_LIMIT_EXCEEDED = 3



_NAMEANDTYPE = _descriptor.Descriptor(
  name='NameAndType',
  full_name='tecton_proto.api.feature_service.NameAndType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tecton_proto.api.feature_service.NameAndType.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='name', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_type', full_name='tecton_proto.api.feature_service.NameAndType.data_type', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='dataType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='tecton_proto.api.feature_service.NameAndType.type', index=2,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', json_name='type', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=294,
  serialized_end=501,
)


_GETFEATURESRESULT = _descriptor.Descriptor(
  name='GetFeaturesResult',
  full_name='tecton_proto.api.feature_service.GetFeaturesResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='features', full_name='tecton_proto.api.feature_service.GetFeaturesResult.features', index=0,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='features', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='join_keys', full_name='tecton_proto.api.feature_service.GetFeaturesResult.join_keys', index=1,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='joinKeys', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=504,
  serialized_end=652,
)


_FEATURESERVERCOMPLEXDATATYPE = _descriptor.Descriptor(
  name='FeatureServerComplexDataType',
  full_name='tecton_proto.api.feature_service.FeatureServerComplexDataType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='tecton_proto.api.feature_service.FeatureServerComplexDataType.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='type', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='element_type', full_name='tecton_proto.api.feature_service.FeatureServerComplexDataType.element_type', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='elementType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=655,
  serialized_end=861,
)


_SLOINFO = _descriptor.Descriptor(
  name='SloInfo',
  full_name='tecton_proto.api.feature_service.SloInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='slo_eligible', full_name='tecton_proto.api.feature_service.SloInfo.slo_eligible', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sloEligible', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='slo_server_time_seconds', full_name='tecton_proto.api.feature_service.SloInfo.slo_server_time_seconds', index=1,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sloServerTimeSeconds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='slo_ineligibility_reasons', full_name='tecton_proto.api.feature_service.SloInfo.slo_ineligibility_reasons', index=2,
      number=2, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sloIneligibilityReasons', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dynamodb_response_size_bytes', full_name='tecton_proto.api.feature_service.SloInfo.dynamodb_response_size_bytes', index=3,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='dynamodbResponseSizeBytes', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='server_time_seconds', full_name='tecton_proto.api.feature_service.SloInfo.server_time_seconds', index=4,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='serverTimeSeconds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='store_time_seconds', full_name='tecton_proto.api.feature_service.SloInfo.store_time_seconds', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='storeTimeSeconds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='store_max_latency', full_name='tecton_proto.api.feature_service.SloInfo.store_max_latency', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='storeMaxLatency', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='store_response_size_bytes', full_name='tecton_proto.api.feature_service.SloInfo.store_response_size_bytes', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='storeResponseSizeBytes', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=864,
  serialized_end=1343,
)


_METADATA_ITEM = _descriptor.Descriptor(
  name='Item',
  full_name='tecton_proto.api.feature_service.Metadata.Item',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tecton_proto.api.feature_service.Metadata.Item.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='name', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='effective_time', full_name='tecton_proto.api.feature_service.Metadata.Item.effective_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='effectiveTime', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='tecton_proto.api.feature_service.Metadata.Item.type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', json_name='type', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data_type', full_name='tecton_proto.api.feature_service.Metadata.Item.data_type', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='dataType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1625,
  serialized_end=1892,
)

_METADATA = _descriptor.Descriptor(
  name='Metadata',
  full_name='tecton_proto.api.feature_service.Metadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='features', full_name='tecton_proto.api.feature_service.Metadata.features', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='features', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='join_keys', full_name='tecton_proto.api.feature_service.Metadata.join_keys', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='joinKeys', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='slo_info', full_name='tecton_proto.api.feature_service.Metadata.slo_info', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sloInfo', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='partial_results', full_name='tecton_proto.api.feature_service.Metadata.partial_results', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='partialResults', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_METADATA_ITEM, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1346,
  serialized_end=1892,
)


_GETFEATURESRESPONSE = _descriptor.Descriptor(
  name='GetFeaturesResponse',
  full_name='tecton_proto.api.feature_service.GetFeaturesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='tecton_proto.api.feature_service.GetFeaturesResponse.result', index=0,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='result', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='tecton_proto.api.feature_service.GetFeaturesResponse.metadata', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='metadata', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1895,
  serialized_end=2065,
)


_GETFEATURESBATCHRESPONSE = _descriptor.Descriptor(
  name='GetFeaturesBatchResponse',
  full_name='tecton_proto.api.feature_service.GetFeaturesBatchResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='tecton_proto.api.feature_service.GetFeaturesBatchResponse.result', index=0,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='result', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='tecton_proto.api.feature_service.GetFeaturesBatchResponse.metadata', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='metadata', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2068,
  serialized_end=2243,
)


_SERVICEMETADATAREQUEST = _descriptor.Descriptor(
  name='ServiceMetadataRequest',
  full_name='tecton_proto.api.feature_service.ServiceMetadataRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='params', full_name='tecton_proto.api.feature_service.ServiceMetadataRequest.params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='params', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2245,
  serialized_end=2350,
)


_SERVICEMETADATARESPONSE = _descriptor.Descriptor(
  name='ServiceMetadataResponse',
  full_name='tecton_proto.api.feature_service.ServiceMetadataResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_service_type', full_name='tecton_proto.api.feature_service.ServiceMetadataResponse.feature_service_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureServiceType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='input_join_keys', full_name='tecton_proto.api.feature_service.ServiceMetadataResponse.input_join_keys', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='inputJoinKeys', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='input_request_context_keys', full_name='tecton_proto.api.feature_service.ServiceMetadataResponse.input_request_context_keys', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='inputRequestContextKeys', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output_join_keys', full_name='tecton_proto.api.feature_service.ServiceMetadataResponse.output_join_keys', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='outputJoinKeys', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_values', full_name='tecton_proto.api.feature_service.ServiceMetadataResponse.feature_values', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureValues', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2353,
  serialized_end=2852,
)


_QUERYFEATURESREQUEST = _descriptor.Descriptor(
  name='QueryFeaturesRequest',
  full_name='tecton_proto.api.feature_service.QueryFeaturesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='params', full_name='tecton_proto.api.feature_service.QueryFeaturesRequest.params', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='params', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2854,
  serialized_end=2957,
)


_QUERYFEATURESRESPONSE = _descriptor.Descriptor(
  name='QueryFeaturesResponse',
  full_name='tecton_proto.api.feature_service.QueryFeaturesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='results', full_name='tecton_proto.api.feature_service.QueryFeaturesResponse.results', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='results', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='tecton_proto.api.feature_service.QueryFeaturesResponse.metadata', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='metadata', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2960,
  serialized_end=3134,
)


_FEATURESERVICELOCATOR = _descriptor.Descriptor(
  name='FeatureServiceLocator',
  full_name='tecton_proto.api.feature_service.FeatureServiceLocator',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_service_id', full_name='tecton_proto.api.feature_service.FeatureServiceLocator.feature_service_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureServiceId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_service_name', full_name='tecton_proto.api.feature_service.FeatureServiceLocator.feature_service_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureServiceName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_package_id', full_name='tecton_proto.api.feature_service.FeatureServiceLocator.feature_package_id', index=2,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featurePackageId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_package_name', full_name='tecton_proto.api.feature_service.FeatureServiceLocator.feature_package_name', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featurePackageName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view_id', full_name='tecton_proto.api.feature_service.FeatureServiceLocator.feature_view_id', index=4,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_view_name', full_name='tecton_proto.api.feature_service.FeatureServiceLocator.feature_view_name', index=5,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureViewName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workspace_name', full_name='tecton_proto.api.feature_service.FeatureServiceLocator.workspace_name', index=6,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workspaceName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'\222A4:2{\"feature_service_name\": \"ctr_prediction_service\"}',
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='locator', full_name='tecton_proto.api.feature_service.FeatureServiceLocator.locator',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=3137,
  serialized_end=3555,
)


_GETFEATURESERVICESTATEREQUEST = _descriptor.Descriptor(
  name='GetFeatureServiceStateRequest',
  full_name='tecton_proto.api.feature_service.GetFeatureServiceStateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='feature_service_locator', full_name='tecton_proto.api.feature_service.GetFeatureServiceStateRequest.feature_service_locator', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureServiceLocator', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3558,
  serialized_end=3702,
)


_GETFEATURESERVICESTATERESPONSE = _descriptor.Descriptor(
  name='GetFeatureServiceStateResponse',
  full_name='tecton_proto.api.feature_service.GetFeatureServiceStateResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3704,
  serialized_end=3742,
)

_NAMEANDTYPE.fields_by_name['data_type'].message_type = _FEATURESERVERCOMPLEXDATATYPE
_NAMEANDTYPE.fields_by_name['type'].enum_type = _FEATURESERVERDATATYPE
_GETFEATURESRESULT.fields_by_name['features'].message_type = google_dot_protobuf_dot_struct__pb2._VALUE
_GETFEATURESRESULT.fields_by_name['join_keys'].message_type = google_dot_protobuf_dot_struct__pb2._VALUE
_FEATURESERVERCOMPLEXDATATYPE.fields_by_name['type'].enum_type = _FEATURESERVERDATATYPE
_FEATURESERVERCOMPLEXDATATYPE.fields_by_name['element_type'].message_type = _FEATURESERVERCOMPLEXDATATYPE
_SLOINFO.fields_by_name['slo_ineligibility_reasons'].enum_type = _SLOINELIGIBILITYREASON
_METADATA_ITEM.fields_by_name['effective_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_METADATA_ITEM.fields_by_name['type'].enum_type = _FEATURESERVERDATATYPE
_METADATA_ITEM.fields_by_name['data_type'].message_type = _FEATURESERVERCOMPLEXDATATYPE
_METADATA_ITEM.containing_type = _METADATA
_METADATA.fields_by_name['features'].message_type = _METADATA_ITEM
_METADATA.fields_by_name['join_keys'].message_type = _METADATA_ITEM
_METADATA.fields_by_name['slo_info'].message_type = _SLOINFO
_GETFEATURESRESPONSE.fields_by_name['result'].message_type = _GETFEATURESRESULT
_GETFEATURESRESPONSE.fields_by_name['metadata'].message_type = _METADATA
_GETFEATURESBATCHRESPONSE.fields_by_name['result'].message_type = _GETFEATURESRESULT
_GETFEATURESBATCHRESPONSE.fields_by_name['metadata'].message_type = _METADATA
_SERVICEMETADATAREQUEST.fields_by_name['params'].message_type = _FEATURESERVICELOCATOR
_SERVICEMETADATARESPONSE.fields_by_name['feature_service_type'].enum_type = _FEATURESERVICETYPE
_SERVICEMETADATARESPONSE.fields_by_name['input_join_keys'].message_type = _NAMEANDTYPE
_SERVICEMETADATARESPONSE.fields_by_name['input_request_context_keys'].message_type = _NAMEANDTYPE
_SERVICEMETADATARESPONSE.fields_by_name['output_join_keys'].message_type = _NAMEANDTYPE
_SERVICEMETADATARESPONSE.fields_by_name['feature_values'].message_type = _NAMEANDTYPE
_QUERYFEATURESREQUEST.fields_by_name['params'].message_type = tecton__proto_dot_api_dot_featureservice_dot_feature__service__request__pb2._GETFEATURESPARAMETERS
_QUERYFEATURESRESPONSE.fields_by_name['results'].message_type = _GETFEATURESRESULT
_QUERYFEATURESRESPONSE.fields_by_name['metadata'].message_type = _METADATA
_FEATURESERVICELOCATOR.oneofs_by_name['locator'].fields.append(
  _FEATURESERVICELOCATOR.fields_by_name['feature_service_id'])
_FEATURESERVICELOCATOR.fields_by_name['feature_service_id'].containing_oneof = _FEATURESERVICELOCATOR.oneofs_by_name['locator']
_FEATURESERVICELOCATOR.oneofs_by_name['locator'].fields.append(
  _FEATURESERVICELOCATOR.fields_by_name['feature_service_name'])
_FEATURESERVICELOCATOR.fields_by_name['feature_service_name'].containing_oneof = _FEATURESERVICELOCATOR.oneofs_by_name['locator']
_FEATURESERVICELOCATOR.oneofs_by_name['locator'].fields.append(
  _FEATURESERVICELOCATOR.fields_by_name['feature_package_id'])
_FEATURESERVICELOCATOR.fields_by_name['feature_package_id'].containing_oneof = _FEATURESERVICELOCATOR.oneofs_by_name['locator']
_FEATURESERVICELOCATOR.oneofs_by_name['locator'].fields.append(
  _FEATURESERVICELOCATOR.fields_by_name['feature_package_name'])
_FEATURESERVICELOCATOR.fields_by_name['feature_package_name'].containing_oneof = _FEATURESERVICELOCATOR.oneofs_by_name['locator']
_FEATURESERVICELOCATOR.oneofs_by_name['locator'].fields.append(
  _FEATURESERVICELOCATOR.fields_by_name['feature_view_id'])
_FEATURESERVICELOCATOR.fields_by_name['feature_view_id'].containing_oneof = _FEATURESERVICELOCATOR.oneofs_by_name['locator']
_FEATURESERVICELOCATOR.oneofs_by_name['locator'].fields.append(
  _FEATURESERVICELOCATOR.fields_by_name['feature_view_name'])
_FEATURESERVICELOCATOR.fields_by_name['feature_view_name'].containing_oneof = _FEATURESERVICELOCATOR.oneofs_by_name['locator']
_GETFEATURESERVICESTATEREQUEST.fields_by_name['feature_service_locator'].message_type = _FEATURESERVICELOCATOR
DESCRIPTOR.message_types_by_name['NameAndType'] = _NAMEANDTYPE
DESCRIPTOR.message_types_by_name['GetFeaturesResult'] = _GETFEATURESRESULT
DESCRIPTOR.message_types_by_name['FeatureServerComplexDataType'] = _FEATURESERVERCOMPLEXDATATYPE
DESCRIPTOR.message_types_by_name['SloInfo'] = _SLOINFO
DESCRIPTOR.message_types_by_name['Metadata'] = _METADATA
DESCRIPTOR.message_types_by_name['GetFeaturesResponse'] = _GETFEATURESRESPONSE
DESCRIPTOR.message_types_by_name['GetFeaturesBatchResponse'] = _GETFEATURESBATCHRESPONSE
DESCRIPTOR.message_types_by_name['ServiceMetadataRequest'] = _SERVICEMETADATAREQUEST
DESCRIPTOR.message_types_by_name['ServiceMetadataResponse'] = _SERVICEMETADATARESPONSE
DESCRIPTOR.message_types_by_name['QueryFeaturesRequest'] = _QUERYFEATURESREQUEST
DESCRIPTOR.message_types_by_name['QueryFeaturesResponse'] = _QUERYFEATURESRESPONSE
DESCRIPTOR.message_types_by_name['FeatureServiceLocator'] = _FEATURESERVICELOCATOR
DESCRIPTOR.message_types_by_name['GetFeatureServiceStateRequest'] = _GETFEATURESERVICESTATEREQUEST
DESCRIPTOR.message_types_by_name['GetFeatureServiceStateResponse'] = _GETFEATURESERVICESTATERESPONSE
DESCRIPTOR.enum_types_by_name['FeatureServiceType'] = _FEATURESERVICETYPE
DESCRIPTOR.enum_types_by_name['FeatureServerDataType'] = _FEATURESERVERDATATYPE
DESCRIPTOR.enum_types_by_name['SloIneligibilityReason'] = _SLOINELIGIBILITYREASON
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

NameAndType = _reflection.GeneratedProtocolMessageType('NameAndType', (_message.Message,), {
  'DESCRIPTOR' : _NAMEANDTYPE,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.NameAndType)
  })
_sym_db.RegisterMessage(NameAndType)

GetFeaturesResult = _reflection.GeneratedProtocolMessageType('GetFeaturesResult', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESRESULT,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.GetFeaturesResult)
  })
_sym_db.RegisterMessage(GetFeaturesResult)

FeatureServerComplexDataType = _reflection.GeneratedProtocolMessageType('FeatureServerComplexDataType', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESERVERCOMPLEXDATATYPE,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.FeatureServerComplexDataType)
  })
_sym_db.RegisterMessage(FeatureServerComplexDataType)

SloInfo = _reflection.GeneratedProtocolMessageType('SloInfo', (_message.Message,), {
  'DESCRIPTOR' : _SLOINFO,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.SloInfo)
  })
_sym_db.RegisterMessage(SloInfo)

Metadata = _reflection.GeneratedProtocolMessageType('Metadata', (_message.Message,), {

  'Item' : _reflection.GeneratedProtocolMessageType('Item', (_message.Message,), {
    'DESCRIPTOR' : _METADATA_ITEM,
    '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
    # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.Metadata.Item)
    })
  ,
  'DESCRIPTOR' : _METADATA,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.Metadata)
  })
_sym_db.RegisterMessage(Metadata)
_sym_db.RegisterMessage(Metadata.Item)

GetFeaturesResponse = _reflection.GeneratedProtocolMessageType('GetFeaturesResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESRESPONSE,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.GetFeaturesResponse)
  })
_sym_db.RegisterMessage(GetFeaturesResponse)

GetFeaturesBatchResponse = _reflection.GeneratedProtocolMessageType('GetFeaturesBatchResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESBATCHRESPONSE,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.GetFeaturesBatchResponse)
  })
_sym_db.RegisterMessage(GetFeaturesBatchResponse)

ServiceMetadataRequest = _reflection.GeneratedProtocolMessageType('ServiceMetadataRequest', (_message.Message,), {
  'DESCRIPTOR' : _SERVICEMETADATAREQUEST,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.ServiceMetadataRequest)
  })
_sym_db.RegisterMessage(ServiceMetadataRequest)

ServiceMetadataResponse = _reflection.GeneratedProtocolMessageType('ServiceMetadataResponse', (_message.Message,), {
  'DESCRIPTOR' : _SERVICEMETADATARESPONSE,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.ServiceMetadataResponse)
  })
_sym_db.RegisterMessage(ServiceMetadataResponse)

QueryFeaturesRequest = _reflection.GeneratedProtocolMessageType('QueryFeaturesRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYFEATURESREQUEST,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.QueryFeaturesRequest)
  })
_sym_db.RegisterMessage(QueryFeaturesRequest)

QueryFeaturesResponse = _reflection.GeneratedProtocolMessageType('QueryFeaturesResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYFEATURESRESPONSE,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.QueryFeaturesResponse)
  })
_sym_db.RegisterMessage(QueryFeaturesResponse)

FeatureServiceLocator = _reflection.GeneratedProtocolMessageType('FeatureServiceLocator', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESERVICELOCATOR,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.FeatureServiceLocator)
  })
_sym_db.RegisterMessage(FeatureServiceLocator)

GetFeatureServiceStateRequest = _reflection.GeneratedProtocolMessageType('GetFeatureServiceStateRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESERVICESTATEREQUEST,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.GetFeatureServiceStateRequest)
  })
_sym_db.RegisterMessage(GetFeatureServiceStateRequest)

GetFeatureServiceStateResponse = _reflection.GeneratedProtocolMessageType('GetFeatureServiceStateResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFEATURESERVICESTATERESPONSE,
  '__module__' : 'tecton_proto.api.featureservice.feature_service_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.api.feature_service.GetFeatureServiceStateResponse)
  })
_sym_db.RegisterMessage(GetFeatureServiceStateResponse)


DESCRIPTOR._options = None
_NAMEANDTYPE.fields_by_name['type']._options = None
_METADATA_ITEM.fields_by_name['type']._options = None
_FEATURESERVICELOCATOR._options = None

_FEATURESERVICE = _descriptor.ServiceDescriptor(
  name='FeatureService',
  full_name='tecton_proto.api.feature_service.FeatureService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=4086,
  serialized_end=6544,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetFeatures',
    full_name='tecton_proto.api.feature_service.FeatureService.GetFeatures',
    index=0,
    containing_service=None,
    input_type=tecton__proto_dot_api_dot_featureservice_dot_feature__service__request__pb2._GETFEATURESREQUEST,
    output_type=_GETFEATURESRESPONSE,
    serialized_options=b'\202\323\344\223\002%\" /v1/feature-service/get-features:\001*\222A\307\002\n\013GetFeatures\032FGet Features API to retrieve feature values from Tecton\'s online storeJ?\n\003200\0228\n\002OK\"2\n\020application/json\022\036{\"result\": {\"features\": [15]}}J\256\001\n\003404\022\246\001\n\tNot Found\"\230\001\n\020application/json\022\203\001{\"error\": \"Unknown feature_service_id: my_feature_service\", \"message\": \"Unknown feature_service_id: my_feature_service\", \"code\": 5}',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetFeaturesBatch',
    full_name='tecton_proto.api.feature_service.FeatureService.GetFeaturesBatch',
    index=1,
    containing_service=None,
    input_type=tecton__proto_dot_api_dot_featureservice_dot_feature__service__request__pb2._GETFEATURESBATCHREQUEST,
    output_type=_GETFEATURESBATCHRESPONSE,
    serialized_options=b'\202\323\344\223\002+\"&/v1/feature-service/get-features-batch:\001*\222A\230\004\n\020GetFeaturesBatch\032UBatch get Features API to retrieve feature values in batch from Tecton\'s online storeJ^\n\003200\022W\n\002OK\"Q\n\020application/json\022={\"result\": {\"features\": [[15, 10, 5, 100],[10, 20, 1, 200]]}}J\256\001\n\003404\022\246\001\n\tNot Found\"\230\001\n\020application/json\022\203\001{\"error\": \"Unknown feature_service_id: my_feature_service\", \"message\": \"Unknown feature_service_id: my_feature_service\", \"code\": 5}J\233\001\n\003400\022\223\001\n\013Bad Request\"\203\001\n\020application/json\022o{\"error\": \"Batch size 27 exceeds maximum of 25.\", \"message\": \"Batch size 27 exceeds maximum of 25.\", \"code\": 3}',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='QueryFeatures',
    full_name='tecton_proto.api.feature_service.FeatureService.QueryFeatures',
    index=2,
    containing_service=None,
    input_type=_QUERYFEATURESREQUEST,
    output_type=_QUERYFEATURESRESPONSE,
    serialized_options=b'\202\323\344\223\002\'\"\"/v1/feature-service/query-features:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetFeatureServiceState',
    full_name='tecton_proto.api.feature_service.FeatureService.GetFeatureServiceState',
    index=3,
    containing_service=None,
    input_type=_GETFEATURESERVICESTATEREQUEST,
    output_type=_GETFEATURESERVICESTATERESPONSE,
    serialized_options=b'\202\323\344\223\0022\"-/v1/feature-service/get-feature-service-state:\001*',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ServiceMetadata',
    full_name='tecton_proto.api.feature_service.FeatureService.ServiceMetadata',
    index=4,
    containing_service=None,
    input_type=_SERVICEMETADATAREQUEST,
    output_type=_SERVICEMETADATARESPONSE,
    serialized_options=b'\202\323\344\223\002!\"\034/v1/feature-service/metadata:\001*\222A\367\004\n\010Metadata\0322Get Metadata about the feature service from TectonJ\331\002\n\003200\022\321\002\n\002OK\"\312\002\n\020application/json\022\265\002{\"featureServiceType\": \"DEFAULT\", \"inputJoinKeys\": [{\"name\": \"adId\", \"type\": \"string\"}],\"feature_values\":[{\"name\": \"ad_ground_truth_ctr_performance_7_days\", \"type\": \"float64\"}, {\"name\": \"user_partner_impression_count_7_days\", \"type\": \"float64\"}, {\"name\": \"user_total_ad_frequency_counts\", \"type\": \"int64\"} ] }J\332\001\n\003404\022\322\001\n\tNot Found\"\304\001\n\020application/json\022\257\001{\"error\": \"Unknown feature_service_name: my_feature_service for workspace: prod\", \"message\": \"Unknown feature_service_name: my_feature_service for workspace: prod\", \"code\": 5}',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_FEATURESERVICE)

DESCRIPTOR.services_by_name['FeatureService'] = _FEATURESERVICE

# @@protoc_insertion_point(module_scope)



from typing import Dict
from typing import List
from typing import Optional
from typing import Union

import pandas

from tecton_proto.args.new_transformation_pb2 import NewTransformationArgs as TransformationArgs
from tecton_proto.args.pipeline_pb2 import ConstantNode
from tecton_proto.args.pipeline_pb2 import Input as InputProto
from tecton_proto.args.pipeline_pb2 import PipelineNode
from tecton_proto.data.new_transformation_pb2 import NewTransformation as Transformation

CONSTANT_TYPE = Optional[Union[str, int, float, bool]]


def _make_mode_to_type():
    lookup = {
        "pandas": pandas.DataFrame,
        "python": Dict,
        "pipeline": PipelineNode,
        "spark_sql": str,
        "snowflake_sql": str,
    }
    try:
        import pyspark.sql

        lookup["pyspark"] = pyspark.sql.DataFrame
    except ImportError:
        pass
    return lookup


MODE_TO_TYPE_LOOKUP = _make_mode_to_type()


def constant_node_to_value(constant_node: ConstantNode) -> CONSTANT_TYPE:
    if constant_node.HasField("string_const"):
        return constant_node.string_const
    elif constant_node.HasField("int_const"):
        return int(constant_node.int_const)
    elif constant_node.HasField("float_const"):
        return float(constant_node.float_const)
    elif constant_node.HasField("bool_const"):
        return constant_node.bool_constant
    elif constant_node.HasField("null_const"):
        return None
    raise KeyError(f"Unknown ConstantNode type: {constant_node}")


def get_keyword_inputs(transformation_node) -> Dict[str, InputProto]:
    """Returns the keyword inputs of transformation_node in a dict."""
    return {
        node_input.arg_name: node_input for node_input in transformation_node.inputs if node_input.HasField("arg_name")
    }


def positional_inputs(transformation_node) -> List[InputProto]:
    """Returns the positional inputs of transformation_node in order."""
    return [node_input for node_input in transformation_node.inputs if node_input.HasField("arg_index")]


def get_transformation_name(transformation: Union[Transformation, TransformationArgs]) -> str:
    if isinstance(transformation, Transformation):
        return transformation.fco_metadata.name
    elif isinstance(transformation, TransformationArgs):
        return transformation.info.name
    else:
        # should ideally never be thrown
        raise Exception(f"Invalid type (expected Transformation or TransformationArgs): {type(transformation)}")


def transformation_type_checker(object_name, result, mode, supported_modes):
    possible_mode = None
    for candidate_mode, candidate_type in MODE_TO_TYPE_LOOKUP.items():
        if isinstance(result, candidate_type):
            possible_mode = candidate_mode
            break
    expected_type = MODE_TO_TYPE_LOOKUP[mode]
    actual_type = type(result)

    if isinstance(result, expected_type):
        return
    elif possible_mode is not None and possible_mode in supported_modes:
        raise TypeError(
            f"Transformation function {object_name} with mode '{mode}' is expected to return result with type {expected_type}, but returns result with type {actual_type} instead. Did you mean to set mode='{possible_mode}'?"
        )
    else:
        raise TypeError(
            f"Transformation function {object_name} with mode {mode} is expected to return result with type {expected_type}, but returns result with type {actual_type} instead."
        )



from typing import Any
from typing import Dict
from typing import Optional
from typing import Union

import pandas
import pendulum
import pyspark

import tecton.interactive.data_frame
import tecton_spark
from tecton._internals import errors
from tecton._internals import utils
from tecton._internals.sdk_decorators import sdk_public_method
from tecton._internals.spark_utils import get_or_create_spark_session
from tecton_spark import logger as logger_lib
from tecton_spark.feature_definition_wrapper import FeatureDefinitionWrapper as FeatureDefinition


logger = logger_lib.get_logger("TectonContext")


class TectonContext:
    """
    Execute Spark SQL queries; access various utils.
    """

    _current_context_instance = None
    _config: Dict[str, Any] = {}

    def __init__(self, spark):
        self._spark = spark

    @classmethod
    def _set_config(cls, custom_spark_options=None):
        """
        Sets the configs for TectonContext instance.
        To take effect it must be called before any calls to TectonContext.get_instance().

        :param custom_spark_options: If spark session gets created by TectonContext, custom spark options/
        """
        cls._config = {"custom_spark_options": custom_spark_options}

    @classmethod
    @sdk_public_method
    def get_instance(cls) -> "TectonContext":
        """
        Get the singleton instance of TectonContext.
        """
        # If the instance doesn't exist, creates a new TectonContext from
        # an existing Spark context. Alternatively, creates a new Spark context on the fly.
        if cls._current_context_instance is not None:
            return cls._current_context_instance
        else:
            return cls._generate_and_set_new_instance()

    @classmethod
    def _generate_and_set_new_instance(cls) -> "TectonContext":
        logger.debug(f"Generating new Spark session")
        spark = get_or_create_spark_session(
            cls._config.get("custom_spark_options"),
        )
        cls._current_context_instance = cls(spark)
        return cls._current_context_instance

    def _list_tables(self):
        """
        Lists the registered tables.

        :return: An array of tables.
        """
        tables = []
        spark_databases = self._spark.catalog.listDatabases()

        for db in spark_databases:
            spark_tables = self._spark.catalog.listTables(db.name)
            for t in spark_tables:
                if db.name == "default":
                    tables.append(t.name)
                else:
                    tables.append("{database}.{table}".format(database=db.name, table=t.name))
        return tables

    # A user-facing validation method
    @classmethod
    def validate_spine_type(cls, spine: Union[pyspark.sql.dataframe.DataFrame, pandas.DataFrame, None]):
        if not isinstance(spine, (pyspark.sql.dataframe.DataFrame, pandas.DataFrame, type(None))):
            raise errors.INVALID_SPINE_TYPE(type(spine))

    # Validates and returns a spark df constructed from the spine.
    # This is an internal method and supports more types than what we support in the user-facing
    # methods, see validate_spine_type
    def _get_spine_df(
        self,
        spine: Union[str, dict, pyspark.sql.dataframe.DataFrame, list, pandas.DataFrame, pyspark.RDD, None],
    ):
        df = None

        # sql
        if isinstance(spine, str):
            df = self._spark.sql(spine)
        # entity dict
        elif isinstance(spine, dict):
            # TODO(there is probably a much more efficient way to have spark join against temporary single row tables)
            df = self._spark.createDataFrame(pandas.DataFrame([spine]))
        elif isinstance(spine, tecton.interactive.data_frame.DataFrame):
            df = spine.to_spark()
        elif isinstance(spine, pyspark.sql.dataframe.DataFrame):
            df = spine
        elif isinstance(spine, (pandas.DataFrame, list, pyspark.RDD)):
            df = self._spark.createDataFrame(spine)
        elif spine is None:
            pass
        else:
            raise errors.INVALID_SPINE_TYPE(type(spine))
        return df

    @sdk_public_method
    def execute(
        self,
        spine: Union[str, dict, pyspark.sql.dataframe.DataFrame, list, pandas.DataFrame, pyspark.RDD, None],
        feature_set_config: Optional["tecton_spark.feature_set_config.FeatureSetConfig"] = None,
        timestamp_key: Optional[str] = None,
        include_feature_view_timestamp_columns: bool = False,
        use_materialized_data: bool = True,
    ):
        """
        Runs a SQL query against Tecton's registered data sources.

        :param spine: Instance of [str, dict, pyspark.sql.dataframe.DataFrame, pandas.DataFrame, list, pyspark.RDD, NoneType].
            `str` = SQL;
            `dict` = Creates a fake table of just one row with the keys as columns;
            `pyspark.sql.dataframe.DataFrame` = instance of spark DataFrame;
            `pandas.DataFrame`, `list`, and `pyspark.RDD` are passed directly to :func:`~pyspark.sql.SQLContext.createDataFrame`
            `NoneType` = No spine, which means extracting all features from the given feature_set_config.
        :param feature_set_config: (Optional) Instance of :class:`~tecton.FeatureSetConfig`.
        :param timestamp_key: (Optional) Name of the time column in the returning Dataframe. The column must be of type Spark timestamp.
            If spine is provided, needs to match time key in it.
            If spine is not provided, we default to one of the underlying feature definition's time column name.
        :param include_feature_view_timestamp_columns: (Optional) Include timestamp columns for every individual FeatureView/FeatureTable.
        :param use_materialized_data: (Optional) Use materialized data if materialization is enabled.
        :return: A Tecton DataFrame.
        """

        df = self._get_spine_df(spine)
        utils.validate_spine_dataframe(df, timestamp_key)

        from tecton._internals import data_frame_helper

        if feature_set_config is not None:
            if spine is not None:
                df = data_frame_helper.get_features_for_spine(
                    self._spark,
                    df,
                    feature_set_config,
                    timestamp_key=timestamp_key,
                    from_source=not use_materialized_data,
                    include_feature_view_timestamp_columns=include_feature_view_timestamp_columns,
                )
            else:
                df = data_frame_helper.get_features_for_preview(
                    self._spark,
                    feature_set_config,
                    spine_time_key=timestamp_key,
                    use_materialized_data=use_materialized_data,
                    include_feature_view_timestamp_columns=include_feature_view_timestamp_columns,
                )
            # Equivalent to df.explain(extended=True) but returns string instead of printing it
            query_execution_plan = df._jdf.queryExecution().toString()
            logger.log(logger_lib.TRACE, f"Query execution plan for FeatureSetConfig:\n{query_execution_plan}")

        df = utils.filter_internal_columns(df)
        return tecton.interactive.data_frame.DataFrame._create(df)

    def _register_temp_views_for_feature_view(
        self,
        fd: FeatureDefinition,
        register_stream=False,
        raw_data_time_limits: Optional[pendulum.Period] = None,
    ):
        for ds in fd.data_sources:
            self._register_temp_view_for_data_source(
                ds, register_stream=register_stream, raw_data_time_limits=raw_data_time_limits
            )

    def _register_temp_view_for_data_source(
        self,
        data_source_proto,
        register_stream=False,
        raw_data_time_limits: Optional[pendulum.Period] = None,
    ):
        from tecton_spark.data_source_helper import register_temp_view_for_data_source

        name = data_source_proto.fco_metadata.name

        register_temp_view_for_data_source(
            self._spark,
            data_source_proto,
            register_stream=register_stream,
            raw_data_time_limits=raw_data_time_limits,
            name=name,
        )

    def _get_spark(self):
        return self._spark



import inspect
import threading
from functools import wraps
from typing import Optional

import pendulum
import pyspark
from typeguard import typechecked

from tecton._internals import errors
from tecton._internals import metadata_service
from tecton._internals.analytics import AnalyticsLogger
from tecton._internals.tecton_errors import TectonAPIInaccessibleError
from tecton._internals.tecton_errors import TectonAPIValidationError
from tecton.tecton_errors import TectonInternalError
from tecton.tecton_errors import TectonValidationError
from tecton_spark import errors as tecton_spark_errors
from tecton_spark import logger
from tecton_spark.id_helper import IdHelper

VERBOSE = True

analytics = AnalyticsLogger()
thread_local_data = threading.local()
sdk_public_method_decorator_enabled = True


def disable_sdk_public_method_decorator():
    global sdk_public_method_decorator_enabled
    sdk_public_method_decorator_enabled = False


def documented_by(func):
    """Normally you should just use functools.wraps over this, but wraps() doesn't work nicely
    when you try to wrap a classmethod in a normal func.
    """

    def wrapper(target):
        target.__doc__ = func.__doc__
        return target

    return wrapper


def sdk_public_method(func):
    arg_names = _get_arg_names(func)

    @wraps(func)
    def _sdk_public_method_wrapper(*args, **kwargs):
        if not sdk_public_method_decorator_enabled:
            return func(*args, **kwargs)

        if not hasattr(thread_local_data, "in_tecton_sdk_public_method_wrapper"):
            thread_local_data.in_tecton_sdk_public_method_wrapper = False

        already_in_wrapper = thread_local_data.in_tecton_sdk_public_method_wrapper
        if already_in_wrapper:
            return func(*args, **kwargs)

        try:
            thread_local_data.in_tecton_sdk_public_method_wrapper = True
            return _invoke_and_transform_errors(
                func, args, kwargs, arg_names, analytics is not None, not already_in_wrapper, typecheck=True
            )
        finally:
            thread_local_data.in_tecton_sdk_public_method_wrapper = already_in_wrapper

    return _sdk_public_method_wrapper


def _is_method(func):
    params = inspect.signature(func).parameters
    return "self" in params or "cls" in params


def _get_arg_names(func):
    arg_names = []
    for i, param in enumerate(inspect.signature(func).parameters.values()):
        name = param.name
        if i == 0 and _is_method(func):
            continue
        arg_names.append(name)
    return arg_names


def _invoke_and_transform_errors(
    func, args, kwargs, arg_names, log_analytics: bool, is_top_level: bool, typecheck: bool = False
):
    original_exception: Optional[Exception] = None
    exception_to_throw: Optional[BaseException] = None
    return_value = None
    start_time = pendulum.now("UTC")

    if typecheck:
        func = typechecked(func)

    if _is_method(func):
        method_object = args[0]
        method_args = args[1:]
    else:
        method_object = None
        method_args = args

    trace_id = IdHelper.generate_string_id()

    if log_analytics:
        analytics.log_method_entry(trace_id, method_object, func, method_args, kwargs, arg_names)

    metadata_service.set_trace_id(trace_id)

    try:
        return_value = func(*args, **kwargs)
    except TectonValidationError as e:
        original_exception = e
        exception_to_throw = e
    except tecton_spark_errors.TectonValidationError as e:
        original_exception = e
        exception_to_throw = e
    except tecton_spark_errors.AccessError as e:
        original_exception = e
        exception_to_throw = e
    except TectonAPIValidationError as e:
        original_exception = e
        exception_to_throw = errors.VALIDATION_ERROR_FROM_MDS(str(e), trace_id)
    except TectonAPIInaccessibleError as e:
        original_exception = e
        exception_to_throw = errors.MDS_INACCESSIBLE(metadata_service._get_host_port())
    except TectonInternalError as e:
        original_exception = e
        exception_to_throw = e
    except Exception as e:
        original_exception = e
        # Skip wrapping some error types that are useful for debugging
        # Top level TypeError is thrown on missing required arguments to the public methods
        # pyspark.sql.utils.CapturedException wraps bunch of Spark errors parsing SQL and building plan:
        # https://github.com/apache/spark/blob/master/python/pyspark/sql/utils.py
        # and is useful for debugging SQL statements
        if (
            logger.get_logging_level() < logger.DEFAULT_LOGGING_LEVEL
            or (isinstance(e, TypeError) and is_top_level)
            or isinstance(e, pyspark.sql.utils.CapturedException)
            or isinstance(e, pendulum.parsing.exceptions.ParserError)
            or isinstance(e, PermissionError)
        ):
            exception_to_throw = e
        else:
            exception_to_throw = errors.INTERNAL_ERROR(str(e))
    finally:
        metadata_service.set_trace_id(None)

    end_time = pendulum.now("UTC")
    execution_time = end_time - start_time

    if log_analytics:
        analytics.log_method_return(trace_id, method_object, func, return_value, execution_time, original_exception)

    if exception_to_throw:
        if not VERBOSE and logger.get_logging_level() >= logger.DEFAULT_LOGGING_LEVEL:
            exception_to_throw = exception_to_throw.with_traceback(None)
            raise exception_to_throw
        else:
            raise exception_to_throw from original_exception

    return return_value



from typing import Dict
from typing import Optional
from typing import Tuple

from pyspark.sql.types import ArrayType
from pyspark.sql.types import BooleanType
from pyspark.sql.types import DataType
from pyspark.sql.types import DoubleType
from pyspark.sql.types import FloatType
from pyspark.sql.types import IntegerType
from pyspark.sql.types import LongType
from pyspark.sql.types import StringType
from pyspark.sql.types import StructType
from pyspark.sql.types import TimestampType

from tecton_proto.common import column_type_pb2
from tecton_proto.common.schema_pb2 import Schema as SchemaProto
from tecton_spark.schema import Schema

# Keep in sync with SchemaUtils.kt
SPARK_TO_TECTON_TYPE = {
    "integer": column_type_pb2.COLUMN_TYPE_INT64,
    "long": column_type_pb2.COLUMN_TYPE_INT64,
    "float": column_type_pb2.COLUMN_TYPE_DOUBLE,
    "double": column_type_pb2.COLUMN_TYPE_DOUBLE,
    "string": column_type_pb2.COLUMN_TYPE_STRING,
    "boolean": column_type_pb2.COLUMN_TYPE_BOOL,
}

# Keep in sync with SchemaUtils.kt
SPARK_ARRAY_ELEMENT_TYPE_TO_TECTON_TYPES = {
    "long": ("long_array", column_type_pb2.COLUMN_TYPE_INT64_ARRAY),
    "float": ("float_array", column_type_pb2.COLUMN_TYPE_FLOAT_ARRAY),
    "double": ("double_array", column_type_pb2.COLUMN_TYPE_DOUBLE_ARRAY),
    "string": ("string_array", column_type_pb2.COLUMN_TYPE_STRING_ARRAY),
}

RAW_SPARK_TYPE_TO_SPARK_DATA_TYPE = {
    "timestamp": TimestampType(),
    "integer": IntegerType(),
    "long": LongType(),
    "float": FloatType(),
    "double": DoubleType(),
    "string": StringType(),
    "boolean": BooleanType(),
    "long_array": ArrayType(LongType()),
    "float_array": ArrayType(FloatType()),
    "double_array": ArrayType(DoubleType()),
    "string_array": ArrayType(StringType()),
}


def schema_from_spark(spark_schema: StructType) -> Schema:
    proto = SchemaProto()
    spark_dict = spark_schema.jsonValue()
    for field in spark_dict["fields"]:
        name = field["name"]
        column = proto.columns.add()
        column.name = name
        column_type, tecton_type = _tecton_type_from_spark_type(field["type"])

        if tecton_type:
            column.feature_server_type = tecton_type
        elif column_type == "timestamp":
            # Timestamp is an exception because it is required for the FV definition
            # but does not have a ColumnType because it's not written in materialized data.
            pass
        else:
            raise ValueError(f"Unsupported Spark type: '{column_type}' of column '{name}'")
        column.raw_spark_type = column_type
    return Schema(proto)


def _tecton_type_from_spark_type(spark_type: Dict) -> Tuple[str, Optional[int]]:
    if not isinstance(spark_type, dict):
        return spark_type, SPARK_TO_TECTON_TYPE.get(spark_type, None)

    if spark_type["type"] != "array":
        return spark_type, None

    return SPARK_ARRAY_ELEMENT_TYPE_TO_TECTON_TYPES.get(spark_type["elementType"], (spark_type, None))


def _spark_data_type_from_raw_spark_type(raw_spark_type: str) -> DataType:
    assert raw_spark_type in RAW_SPARK_TYPE_TO_SPARK_DATA_TYPE, "Unknown raw spark type: " + raw_spark_type
    return RAW_SPARK_TYPE_TO_SPARK_DATA_TYPE[raw_spark_type]


def schema_to_spark(schema: Schema) -> StructType:
    from pyspark.sql.types import StructType

    ret = StructType()
    for col_name, col_spark_data_type in column_name_spark_data_types(schema):
        ret.add(col_name, col_spark_data_type)
    return ret


def column_name_spark_data_types(schema: Schema):
    return [(c[0], _spark_data_type_from_raw_spark_type(c[1])) for c in schema.column_name_raw_spark_types()]



"""Helper module for creating spark data source operations such as creating DataFrames and
registering temp views. Methods within this class should exclusively operate on proto
representations of the data model.
"""
from typing import Dict
from typing import List
from typing import Optional
from typing import Set
from typing import Tuple

import pendulum
from pyspark.sql import DataFrame
from pyspark.sql import functions
from pyspark.sql import SparkSession
from pyspark.sql.functions import coalesce
from pyspark.sql.functions import to_timestamp
from pyspark.sql.types import ArrayType
from pyspark.sql.types import DateType
from pyspark.sql.types import MapType
from pyspark.sql.types import StringType
from pyspark.sql.types import StructField
from pyspark.sql.types import StructType

import tecton_spark.errors_spark
from tecton_proto.args.data_source_config_pb2 import INITIAL_STREAM_POSITION_LATEST
from tecton_proto.args.data_source_config_pb2 import INITIAL_STREAM_POSITION_TRIM_HORIZON
from tecton_proto.args.data_source_config_pb2 import INITIAL_STREAM_POSITION_UNSPECIFIED
from tecton_proto.data.batch_data_source_pb2 import BatchDataSource
from tecton_proto.data.batch_data_source_pb2 import FileDataSourceFormat
from tecton_proto.data.stream_data_source_pb2 import StreamDataSource
from tecton_proto.data.virtual_data_source_pb2 import VirtualDataSource
from tecton_spark import conf
from tecton_spark import function_serialization
from tecton_spark.data_source_credentials import get_kafka_secrets
from tecton_spark.feature_definition_wrapper import FeatureDefinitionWrapper as FeatureDefinition
from tecton_spark.id_helper import IdHelper
from tecton_spark.spark_schema_wrapper import SparkSchemaWrapper

INITIAL_STREAM_POSITION_STR_TO_ENUM = {
    "latest": INITIAL_STREAM_POSITION_LATEST,
    "trim_horizon": INITIAL_STREAM_POSITION_TRIM_HORIZON,
}

INITIAL_STREAM_POSITION_ENUM_TO_STR: Dict[str, Optional[str]] = {
    v: k for k, v in INITIAL_STREAM_POSITION_STR_TO_ENUM.items()
}
INITIAL_STREAM_POSITION_ENUM_TO_STR[INITIAL_STREAM_POSITION_UNSPECIFIED] = None


def _is_running_on_emr() -> bool:
    import os

    return "EMR_RELEASE_LABEL" in os.environ


def _validate_data_source_proto(data_source: BatchDataSource):
    if data_source.HasField("hive_table"):
        assert data_source.hive_table.HasField("database"), "Invalid HiveTableDataSource: no database provided"
        assert data_source.hive_table.HasField("table"), "Invalid HiveTableDataSource: no table provided"
    elif data_source.HasField("file"):
        pass
    elif data_source.HasField("redshift_db"):
        redshift = data_source.redshift_db
        assert redshift.HasField("endpoint"), "Invalid RedshiftDataSource: no endpoint provided"
        assert redshift.HasField("table") or redshift.HasField(
            "query"
        ), "Invalid RedshiftDataSource: no table or query provided"
    elif data_source.HasField("snowflake"):
        snowflake = data_source.snowflake.snowflakeArgs
        required_args = ["url", "database", "schema", "warehouse"]
        for arg in required_args:
            assert snowflake.HasField(arg), f"Invalid SnowflakeDataSource: no {arg} provided"
        assert snowflake.HasField("table") or snowflake.HasField(
            "query"
        ), "Invalid SnowflakeDataSource: no table or query provided"
    else:
        assert False, "BatchDataSource must set hive_table, file, redshift_db, or snowflake"


def _get_raw_hive_table_dataframe(spark: SparkSession, database: str, table: str) -> DataFrame:
    spark.sql("USE {}".format(database))
    return spark.table(table)


def get_raw_dataframe(
    spark: SparkSession, data_source: BatchDataSource, called_for_schema_computation=False
) -> DataFrame:
    """Returns a DataFrame of the raw, untranslated data defined by the given BatchDataSource proto.

    :param spark: Spark session.
    :param data_source: BatchDataSource proto.
    :param called_for_schema_computation: If set, optimizations are applied for faster schema computations.
                                          i.e. FileDSConfig.schema_uri is used to avoid expensive partition discovery.

    :return: The DataFrame for the raw data.
    """
    _validate_data_source_proto(data_source)
    if data_source.HasField("hive_table"):
        df = _get_raw_hive_table_dataframe(spark, data_source.hive_table.database, data_source.hive_table.table)
    elif data_source.HasField("redshift_db"):
        df = get_redshift_dataframe(
            spark,
            data_source.redshift_db.endpoint,
            data_source.redshift_db.temp_s3,
            table=data_source.redshift_db.table,
            query=data_source.redshift_db.query,
        )
    elif data_source.HasField("snowflake"):
        df = get_snowflake_dataframe(
            spark,
            data_source.snowflake.snowflakeArgs.url,
            data_source.snowflake.snowflakeArgs.database,
            data_source.snowflake.snowflakeArgs.schema,
            data_source.snowflake.snowflakeArgs.warehouse,
            data_source.snowflake.snowflakeArgs.role,
            data_source.snowflake.snowflakeArgs.table,
            data_source.snowflake.snowflakeArgs.query,
        )
    else:
        # FileDataSource
        reader = spark.read
        uri = data_source.file.uri
        if called_for_schema_computation and data_source.file.HasField("schema_uri"):
            # Setting basePath includes the path-based partitions in the DataFrame schema.
            # https://spark.apache.org/docs/latest/sql-data-sources-parquet.html#partition-discovery
            reader = reader.option("basePath", data_source.file.uri)
            uri = data_source.file.schema_uri

        if data_source.file.HasField("schema_override"):
            schema = SparkSchemaWrapper.from_proto(data_source.file.schema_override)
            reader = reader.schema(schema.unwrap())

        if data_source.file.format == FileDataSourceFormat.FILE_DATA_SOURCE_FORMAT_JSON:
            action = lambda: reader.json(uri)
        elif data_source.file.format == FileDataSourceFormat.FILE_DATA_SOURCE_FORMAT_PARQUET:
            action = lambda: reader.parquet(uri)
        elif data_source.file.format == FileDataSourceFormat.FILE_DATA_SOURCE_FORMAT_CSV:
            action = lambda: reader.csv(uri, header=True)
        else:
            raise AssertionError(f"Unsupported file format '{data_source.file.format}'")

        df = tecton_spark.errors_spark.handleDataAccessErrors(action, data_source.file.uri)
        if data_source.file.convert_to_glue_format:
            df = convert_json_like_schema_to_glue_format(spark, df)

    return df


def get_table_dataframe(
    spark: SparkSession, data_source: BatchDataSource, called_for_schema_computation=False
) -> DataFrame:
    """Returns a DataFrame for a table defined by given BatchDataSource proto.

    :param spark: Spark session.
    :param data_source: BatchDataSource proto.
    :param called_for_schema_computation: If set, optimizations are applied for faster schema computations.
                                          i.e. FileDSConfig.schema_uri is used to avoid expensive partition discovery.

    :return: The DataFrame created from the data source.
    """
    df = get_raw_dataframe(spark, data_source, called_for_schema_computation)
    if data_source.HasField("raw_batch_translator"):
        translator_fn = function_serialization.from_proto(data_source.raw_batch_translator)
        df = translator_fn(df)
    if data_source.HasField("timestamp_column_properties"):
        ts_format = None
        if data_source.timestamp_column_properties.HasField("format"):
            ts_format = data_source.timestamp_column_properties.format
        df = apply_timestamp_column(df, data_source.timestamp_column_properties.column_name, ts_format)

    return df


def get_redshift_dataframe(
    spark: SparkSession, endpoint: str, temp_s3: str, table: Optional[str] = None, query: Optional[str] = None
) -> DataFrame:
    """Returns a DataFrame for a Redshift table defined by given RedshiftDataSource proto.

    :param table: The table name in redshift
    :param temp_s3: The s3 URI for temp data
    :param endpoint: The connection endpoint for redshift (without user or password)
    :param spark: Spark session.

    :return: The DataFrame created from the data source.
    """

    if _is_running_on_emr():
        spark_format = "io.github.spark_redshift_community.spark.redshift"
    else:
        spark_format = "com.databricks.spark.redshift"

    params = {"user": conf.get_or_none("REDSHIFT_USER"), "password": conf.get_or_none("REDSHIFT_PASSWORD")}
    full_connection_string = f"jdbc:redshift://{endpoint};user={params['user']};password={params['password']}"

    df_reader = (
        spark.read.format(spark_format)
        .option("url", full_connection_string)
        .option("tempdir", temp_s3)
        .option("forward_spark_s3_credentials", "true")
    )

    if table and query:
        raise AssertionError(f"Should only specify one of table and query sources for redshift")
    if not table and not query:
        raise AssertionError(f"Missing both table and query sources for redshift, exactly one must be present")

    if table:
        df_reader = df_reader.option("dbtable", table)
    else:
        df_reader = df_reader.option("query", query)

    df = df_reader.load()
    return df


def get_snowflake_dataframe(
    spark: SparkSession,
    url: str,
    database: str,
    schema: str,
    warehouse: str,
    role: Optional[str] = None,
    table: Optional[str] = None,
    query: Optional[str] = None,
) -> DataFrame:
    """Returns a DataFrame for a Snowflake table defined by given SnowflakeDataSource proto.

    :param spark: Spark session.
    :param url: The table name in Snowflake

    :return: The DataFrame created from the data source.
    """

    if table and query:
        raise AssertionError(f"Should only specify one of table and query sources for Snowflake")
    if not table and not query:
        raise AssertionError(f"Missing both table and query sources for Snowflake, exactly one must be present")

    user = conf.get_or_none("SNOWFLAKE_USER")
    password = conf.get_or_none("SNOWFLAKE_PASSWORD")
    if not user or not password:
        raise AssertionError(
            "Snowflake user and password not configured. Instructions at https://docs.tecton.ai/v2/setting-up-tecton/03g-connecting-snowflake.html"
        )

    options = {
        "sfUrl": url,
        "sfUser": user,
        "sfPassword": password,
        "sfDatabase": database,
        "sfSchema": schema,
        "sfWarehouse": warehouse,
        "APPLICATION": "tecton-ai",
    }

    if role:
        options["sfRole"] = role

    df_reader = spark.read.format("snowflake").options(**options)

    if table:
        df_reader = df_reader.option("dbtable", table)
    else:
        df_reader = df_reader.option("query", query)

    df = df_reader.load()
    return df


def apply_timestamp_column(df: DataFrame, ts_column: str, ts_format: Optional[str]) -> DataFrame:
    # Verify the raw source's timestamp column is of type "string"
    column_names = df.schema.names
    if ts_column not in column_names:
        raise AssertionError(f"Timestamp column '{ts_column}' not found in schema. Found: {column_names}")

    ts_type = df.schema[ts_column].dataType.jsonValue()
    if ts_type != "timestamp":
        assert ts_type == "string", f"Column '{ts_column}' has type '{ts_type}', expected 'string' or 'timestamp'"
        # Apply timestamp transform

        # Here we use coalesce to first try transforming string to timestamp using the user provided format,
        # and if it doesn't work we'll instead let Spark figure it out.
        # Ideally, if the user provided format didn't work, we would not fallback to the Spark default. However, it
        # would be difficult to remove this behavior, and it's hard to imagine a scenario where this would be a problem
        # other than being a bit too "magical". Ref: https://tecton.atlassian.net/browse/TEC-6611
        df = df.withColumn(ts_column, coalesce(to_timestamp(df[ts_column], ts_format), to_timestamp(df[ts_column])))

    return df


def apply_partition_and_timestamp_filter(
    df: DataFrame, data_source: BatchDataSource, raw_data_time_limits: pendulum.Period
) -> DataFrame:
    """Applies a partition and timestamp filters if the respective column names are set.

    :return: The DataFrame with filter applied.
    """

    # backward compatibility. If we wanted to deprecate, this could be replaced by
    # datetime_partition_columns containing [col_name, "%Y-%m-%d", 24 * 60 * 60]
    if data_source.HasField("date_partition_column"):
        date_start = raw_data_time_limits.start.to_date_string()
        date_end = raw_data_time_limits.end.to_date_string()
        date_partition_column = functions.col(data_source.date_partition_column)
        df = df.where((date_start <= date_partition_column) & (date_partition_column <= date_end))
    # add datetime partition filters
    elif len(data_source.datetime_partition_columns) > 0:
        partition_filter = _build_partition_filter(data_source.datetime_partition_columns, raw_data_time_limits)
        df = df.where(partition_filter)
    # add timestamp filter
    if data_source.HasField("timestamp_column_properties"):
        ts_column = functions.col(data_source.timestamp_column_properties.column_name)
        df = df.where((ts_column >= raw_data_time_limits.start) & (ts_column < raw_data_time_limits.end))
    if not data_source.HasField("timestamp_column_properties"):
        raise Exception(f"DataSource {data_source.fco_metadata.name} requires timestamp column name for FeatureView")
    return df


# Generate filter on time_range with at most OR of 2 filters.
# This means we could end up scanning more partitions than necessary, but number extra scanned will be at most
# 3x.
# Worst case: time_range = 367 days across 3 years, we scan entire 3 years.
#             time_range = 365 days, we scan up to 2 years including that 1 year range
#             time_range = 28.1 days, we could scan all of January + February + March = 90 days
#
# 2 cases to consider
# Example: partition cols y, m, d, h: in both examples, the time range is between day and month, so we don't add any
# filters on hour, even though it would be possible to scan some fewer partitions if we did
# (A)Time range: 2020 Jan 10 10:00:00 AM - 2020 Jan 15 5:00:00 AM
#  --> (y = start.year & m = start.month & d >= start.day & d <= end.day)
# (B)Time range: 2019 Dec 21 10:00:00 AM - 2020 Jan 10 5:00:00 AM
#  --> ((y = start.year & m = start.month & d >= start.day) | (y = end.year & m = end.month & d <= end.day))
def _build_partition_filter(datetime_partition_columns, raw_data_time_limits):
    start_time, end_time = raw_data_time_limits.start, raw_data_time_limits.end
    time_range_seconds = raw_data_time_limits.total_seconds()

    # Filters relating to start_time and end_time
    start_filters = []
    end_filters = []
    # Common filter that applies to the entire range
    common_filters = []

    # sort partition columns by the number of seconds they represent from highest to lowest
    partitions_high_to_low = sorted(datetime_partition_columns, key=lambda c: c.minimum_seconds, reverse=True)
    for partition in partitions_high_to_low:
        partition_col = functions.col(partition.column_name)
        partition_value_at_start = _partition_value_for_time(partition, start_time)
        partition_value_at_end = _partition_value_for_time(partition, end_time)
        # If partition's datepart minimum length is >= time_range_seconds, we can be sure that 2 or fewer equality filters are enough to cover all possible times in time_limits
        if partition.minimum_seconds >= time_range_seconds:
            if partition_value_at_start == partition_value_at_end:
                common_filters.append(partition_col == partition_value_at_start)
            else:
                start_filters.append(partition_col == partition_value_at_start)
                end_filters.append(partition_col == partition_value_at_end)
        # Otherwise, we need to use a range filter
        else:
            start_range_filter = partition_col >= partition_value_at_start
            end_range_filter = partition_col <= partition_value_at_end
            # Case A: there are only common filters
            if len(start_filters) == 0:
                common_filters.append(start_range_filter)
                common_filters.append(end_range_filter)
                # we can't combine range filters on multiple columns, so break and ignore any smaller columns
                break
            # Case B
            else:
                start_filters.append(start_range_filter)
                end_filters.append(end_range_filter)
                # we can't combine range filters on multiple columns, so break and ignore any smaller columns
                break

    common_filter = _and_filters_in_list(common_filters)
    start_filter = _and_filters_in_list(start_filters)
    end_filter = _and_filters_in_list(end_filters)

    return common_filter & (start_filter | end_filter)


def _partition_value_for_time(partition, time):
    fmt = partition.format_string
    # On mac/linux strftime supports these formats, but
    # Windows python does not
    # Zero-padded formats are safe for a string comparison. Otherwise we need to compare ints
    # As long as the values returned here are ints, the column will be implicitly converted if needed.
    if fmt == "%-Y":
        return time.year
    elif fmt == "%-m":
        return time.month
    elif fmt == "%-d":
        return time.day
    elif fmt == "%-H":
        return time.hour
    return time.strftime(fmt)


def _and_filters_in_list(filter_list):
    if len(filter_list) == 0:
        return functions.lit(True)
    else:
        from functools import reduce

        return reduce(lambda x, y: x & y, filter_list)


def get_table_column_and_partition_names(spark: SparkSession, data_source: "HiveDSConfig") -> Tuple[Set[str], Set[str]]:  # type: ignore
    """Returns a tuple of (set of column names, set of partition names) for the given BatchDataSource."""
    _validate_data_source_proto(data_source)
    database = data_source.hive_table.database
    table = data_source.hive_table.table
    pandas_df = spark.sql(f"DESCRIBE TABLE {database}.{table}").toPandas()
    column_names = set(pandas_df[~pandas_df.col_name.str.startswith("#")].col_name)
    partition_index = pandas_df.index[pandas_df["col_name"] == "# Partition Information"]
    if partition_index.size > 0:
        partition_info = pandas_df[partition_index[0] :]
        partition_names = set(partition_info[~partition_info.col_name.str.startswith("#")].col_name)
    else:
        partition_names = set()

    return column_names, partition_names


def create_kinesis_stream_reader(
    spark: SparkSession,
    stream_name: str,
    region: str,
    initial_stream_position: Optional[str],
    data_source_options: List[StreamDataSource.Option],
    option_overrides: Optional[Dict[str, str]],
) -> DataFrame:
    """
    Returns a DataFrame representing a Kinesis stream reader.

    :param data_source_options: Spark options specified in the data source definition.
    :param option_overrides: Spark options that should override options set implicitly (e.g. ``stream_name``) or
        explicitly  (e.g. ``options``) by the data source definition.
    """
    options = {"streamName": stream_name}
    if _is_running_on_emr():
        options.update(
            {
                "endpointUrl": f"https://kinesis.{region}.amazonaws.com",
                "kinesis.client.describeShardInterval": "30s",
                "startingPosition": initial_stream_position,
            }
        )
    else:
        options.update({"region": region, "shardFetchInterval": "30s", "initialPosition": initial_stream_position})

    databricks_to_qubole_map = {
        "awsaccesskey": "awsAccessKeyId",
        "rolearn": "awsSTSRoleARN",
        "rolesessionname": "awsSTSSessionName",
    }
    lowercase_data_source_options = {option.key.lower(): option.value for option in data_source_options}
    for option in data_source_options:
        if option.key.lower() in databricks_to_qubole_map and _is_running_on_emr():
            if option.key.lower() == "rolearn" and "rolesessionname" not in lowercase_data_source_options:
                # this field must be supplied if we use roleArn for qubole kinesis reader
                options["awsSTSSessionName"] = "tecton-materialization"
            options[databricks_to_qubole_map[option.key.lower()]] = option.value
        else:
            options[option.key] = option.value

    if option_overrides:
        options.update(option_overrides)

    reader = spark.readStream.format("kinesis").options(**options)
    return reader.load()


def create_kafka_stream_reader(
    spark: SparkSession,
    kafka_bootstrap_servers: str,
    topics: str,
    data_source_options: List[StreamDataSource.Option],
    option_overrides: Optional[Dict[str, str]],
    ssl_keystore_location: Optional[str] = None,
    ssl_keystore_password_secret_id: Optional[str] = None,
    ssl_truststore_location: Optional[str] = None,
    ssl_truststore_password_secret_id: Optional[str] = None,
    security_protocol: Optional[str] = None,
) -> DataFrame:
    """Returns a Kafka stream reader.

    :param data_source_options: Spark options specified in the data source definition.
    :param option_overrides: Spark options that should override options set implicitly (e.g. ``topics``) or explicitly
        (e.g. ``options``) by the data source definition.
    """
    options = {o.key: o.value for o in data_source_options}
    options["kafka.bootstrap.servers"] = kafka_bootstrap_servers
    options["subscribe"] = topics
    if "startingOffsets" not in options:
        # Don't override startingOffsets if it's set explicitly in the data source definition.
        options["startingOffsets"] = "earliest"
    reader = spark.readStream.format("kafka").options(**options)
    if ssl_keystore_location:
        local_keystore_loc, local_keystore_password = get_kafka_secrets(
            ssl_keystore_location, ssl_keystore_password_secret_id
        )
        options["kafka.ssl.keystore.location"] = local_keystore_loc
        if local_keystore_password:
            options["kafka.ssl.keystore.password"] = local_keystore_password
    if ssl_truststore_location:
        local_truststore_loc, local_truststore_password = get_kafka_secrets(
            ssl_truststore_location, ssl_truststore_password_secret_id
        )
        options["kafka.ssl.truststore.location"] = local_truststore_loc
        if local_truststore_password:
            options["kafka.ssl.truststore.password"] = local_truststore_password
    if security_protocol:
        options["kafka.security.protocol"] = security_protocol

    if option_overrides:
        options.update(option_overrides)

    reader = spark.readStream.format("kafka").options(**options)
    return reader.load()


def get_raw_stream_dataframe(
    spark: SparkSession, stream_data_source: StreamDataSource, option_overrides: Optional[Dict[str, str]] = None
) -> DataFrame:
    """Returns a DataFrame representing the raw stream data.

    :param spark: Spark session.
    :param stream_data_source: StreamDataSource proto.
    :param option_overrides: A dictionary of Spark readStream options that will override any readStream options set by
        the data source.

    :return: The DataFrame for the raw stream data source.
    """
    if stream_data_source.HasField("kinesis_data_source"):
        kinesis_data_source = stream_data_source.kinesis_data_source
        initial_stream_position = _get_initial_stream_position(stream_data_source)
        df = create_kinesis_stream_reader(
            spark,
            kinesis_data_source.stream_name,
            kinesis_data_source.region,
            initial_stream_position,
            stream_data_source.options,
            option_overrides,
        )
    elif stream_data_source.HasField("kafka_data_source"):
        kafka_data_source = stream_data_source.kafka_data_source
        df = create_kafka_stream_reader(
            spark,
            kafka_data_source.bootstrap_servers,
            kafka_data_source.topics,
            stream_data_source.options,
            option_overrides,
            ssl_keystore_location=kafka_data_source.ssl_keystore_location,
            ssl_keystore_password_secret_id=kafka_data_source.ssl_keystore_password_secret_id,
            ssl_truststore_location=kafka_data_source.ssl_truststore_location,
            ssl_truststore_password_secret_id=kafka_data_source.ssl_truststore_password_secret_id,
            security_protocol=kafka_data_source.security_protocol,
        )
    else:
        raise ValueError("Unknown stream data source type")
    return df


def get_stream_dataframe(
    spark: SparkSession, stream_data_source: StreamDataSource, option_overrides: Optional[Dict[str, str]] = None
) -> DataFrame:
    """Returns a DataFrame representing a stream data source *without* any options specified.
    Use get_stream_dataframe_with_options to get a DataFrame with stream-specific options.

    :param spark: Spark session.
    :param stream_data_source: StreamDataSource proto.
    :param option_overrides: A dictionary of Spark readStream options that will override any readStream options set by
        the data source.

    :return: The DataFrame created from the data source.
    """

    df = get_raw_stream_dataframe(spark, stream_data_source, option_overrides)
    translator_fn = function_serialization.from_proto(stream_data_source.raw_stream_translator)
    return translator_fn(df)


def get_stream_dataframe_with_options(
    spark: SparkSession, stream_data_source: StreamDataSource, option_overrides: Optional[Dict[str, str]] = None
) -> DataFrame:
    """Returns a DataFrame representing a stream data source with additional options:
        - watermark
        - drop duplicate column names
        - initial stream position

    :param spark: Spark session.
    :param stream_data_source: StreamDataSource proto.
    :param option_overrides: A dictionary of Spark readStream options that will override any readStream options set by
        the data source.

    :return: The DataFrame created from the data source.
    """

    df = get_stream_dataframe(spark, stream_data_source, option_overrides)

    watermark = _get_watermark(stream_data_source)
    df = df.withWatermark(stream_data_source.time_column, watermark)

    dedup_columns = [column for column in stream_data_source.deduplication_column_names]
    if dedup_columns:
        df = df.dropDuplicates(dedup_columns)

    return df


def _get_watermark(stream_data_source: StreamDataSource) -> str:
    """Returns watermark as an "N seconds" string for the streaming data source.

    :param stream_data_source: StreamDataSource proto.

    :return: The watermark duration in seconds in string format.
    :rtype: String
    """
    return "{} seconds".format(stream_data_source.stream_config.watermark_delay_threshold.seconds)


def _get_initial_stream_position(stream_data_source: StreamDataSource) -> Optional[str]:
    """Returns initial stream position as a string (e.g. "latest") for the streaming data source.

    :param stream_data_source: StreamDataSource proto.

    :return: The initial stream position in string format.
    """
    return INITIAL_STREAM_POSITION_ENUM_TO_STR[stream_data_source.stream_config.initial_stream_position]


# should not be invoked directly by public SDK. Use TectonContext wrapper instead
def register_temp_views_for_feature_definition(
    spark: SparkSession,
    feature_definition: FeatureDefinition,
    data_sources: List[VirtualDataSource],
    register_stream: bool,
    raw_data_time_limits: Optional[pendulum.Period] = None,
):
    """Registers spark temp views per DataSource based on a FeatureView proto.

    :param spark: Spark session.
    :param feature_definition: FeatureDefinition interface implementing class.
    :param data_sources: Data source protos
    :param register_stream: If true, a stream DataFrame will be registered for streaming DSs instead of batch.
    :param raw_data_time_limits: The raw data time limits for the batch data source.
    """
    ds_by_id = {IdHelper.to_string(ds.virtual_data_source_id): ds for ds in data_sources}

    for ds_id in feature_definition.data_source_ids:
        register_temp_view_for_data_source(
            spark,
            ds_by_id[ds_id],
            register_stream,
            raw_data_time_limits=raw_data_time_limits,
        )


# should not be invoked directly by public SDK. Use TectonContext wrapper instead
def register_temp_view_for_data_source(
    spark: SparkSession,
    data_source: VirtualDataSource,
    register_stream: bool,
    raw_data_time_limits: Optional[pendulum.Period] = None,
    name: str = None,
    called_for_schema_computation: bool = False,
):
    """Registers a spark temp view for the data source.
    :param spark: Spark session.
    :param data_source: DataSource proto.
    :param register_stream: If true, a stream DataFrame will be registered for streaming DataSources instead of batch.
    :param raw_data_time_limits: If set, a data frame will filter on the time limits based on any partition columns that exist.
    :param name: If set, use this name for the temp view. Defaults to DS name.
    :param called_for_schema_computation: Indicates if method is being invoked to compute a schema.
    """
    df = get_ds_dataframe(
        spark,
        data_source,
        register_stream,
        raw_data_time_limits=raw_data_time_limits,
        called_for_schema_computation=called_for_schema_computation,
    )
    df.createOrReplaceTempView(data_source.fco_metadata.name if name is None else name)


def get_ds_dataframe(
    spark: SparkSession,
    data_source: VirtualDataSource,
    consume_streaming_data_source: bool,
    raw_data_time_limits: Optional[pendulum.Period] = None,
    called_for_schema_computation=False,
):
    assert (
        raw_data_time_limits is None or not consume_streaming_data_source
    ), "Can't specify time limits when consuming streaming data source"

    if consume_streaming_data_source:
        assert data_source.HasField(
            "stream_data_source"
        ), f"Can't consume streaming data source from the BatchDataSource: {data_source.fco_metadata.name}."
        df = get_stream_dataframe_with_options(spark, data_source.stream_data_source, None)
    else:
        df = get_table_dataframe(
            spark, data_source.batch_data_source, called_for_schema_computation=called_for_schema_computation
        )
        if data_source.HasField("stream_data_source"):
            schema = data_source.stream_data_source.spark_schema
            cols = [field.name for field in schema.fields]
            df = df.select(*cols)

        if raw_data_time_limits is not None:
            df = apply_partition_and_timestamp_filter(df, data_source.batch_data_source, raw_data_time_limits)

    return df


def convert_json_like_schema_to_glue_format(spark: SparkSession, df: DataFrame) -> DataFrame:
    """
    Converts a DataFrame schema to lowercase. This assumes JSON so
    MapTypes or Arrays of non-StructTypes are not allowed.

    :param spark: Spark session.
    :param df: DataFrame input.
    :return: DataFrame with lowercase schema.
    """

    def _get_lowercase_schema(datatype):
        if type(datatype) == ArrayType:
            return _get_lowercase_array_schema(datatype)
        elif type(datatype) == StructType:
            return _get_lowercase_structtype_schema(datatype)
        elif type(col.dataType) == MapType:
            raise TypeError("MapType not supported in JSON schema")
        return datatype

    def _get_lowercase_structtype_schema(s) -> StructType:
        assert type(s) == StructType, f"Invalid argument type {type(s)}, expected StructType"
        struct_fields = []
        for col in s:
            datatype = _get_lowercase_schema(col.dataType)
            struct_fields.append(StructField(col.name.lower(), datatype))
        return StructType(struct_fields)

    def _get_lowercase_array_schema(c) -> ArrayType:
        assert (
            type(c.elementType) == StructType
        ), f"Invalid ArrayType element type {type(c)}, expected StructType for valid JSON arrays."
        datatype = c.elementType
        struct_schema = _get_lowercase_structtype_schema(datatype)
        return ArrayType(struct_schema)

    # Simple columns (LongType, StringType, etc) can just be renamed without
    # casting schema.
    # Nested fields within complex columns (ArrayType, StructType) must also be recursively converted
    # to lowercase names, so they must be casted.
    # DateType columns should be converted to StringType to match Glue schemas.
    new_fields = []
    for col in df.schema:
        if type(col.dataType) in [ArrayType, StructType, MapType]:
            t = _get_lowercase_schema(col.dataType)
            new_fields.append(functions.col(col.name).cast(t).alias(col.name.lower()))
        elif type(col.dataType) is DateType:
            new_fields.append(functions.col(col.name).cast(StringType()).alias(col.name.lower()))
        else:
            new_fields.append(functions.col(col.name).alias(col.name.lower()))
    return df.select(new_fields)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/saved_feature_data_frame.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.common import spark_schema_pb2 as tecton__proto_dot_common_dot_spark__schema__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tecton_proto/data/saved_feature_data_frame.proto',
  package='tecton_proto.data',
  syntax='proto2',
  serialized_options=b'\n\017com.tecton.dataP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n0tecton_proto/data/saved_feature_data_frame.proto\x12\x11tecton_proto.data\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1ctecton_proto/common/id.proto\x1a&tecton_proto/common/spark_schema.proto\"\xd1\x01\n\x19SavedFeatureDataFrameInfo\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x39\n\ncreated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x1f\n\x0bis_archived\x18\x03 \x01(\x08R\nisArchived\x12\x1c\n\tworkspace\x18\x04 \x01(\tR\tworkspace\x12&\n\x0fis_data_deleted\x18\x05 \x01(\x08R\risDataDeleted\"\x9f\x06\n\x15SavedFeatureDataFrame\x12T\n\x1asaved_feature_dataframe_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x17savedFeatureDataframeId\x12@\n\x04info\x18\x02 \x01(\x0b\x32,.tecton_proto.data.SavedFeatureDataFrameInfoR\x04info\x12-\n\x12\x64\x61taframe_location\x18\x03 \x01(\tR\x11\x64\x61taframeLocation\x12G\n\x12\x66\x65\x61ture_package_id\x18\x04 \x01(\x0b\x32\x17.tecton_proto.common.IdH\x00R\x10\x66\x65\x61turePackageId\x12G\n\x12\x66\x65\x61ture_service_id\x18\x05 \x01(\x0b\x32\x17.tecton_proto.common.IdH\x00R\x10\x66\x65\x61tureServiceId\x12\x32\n\x14\x66\x65\x61ture_package_name\x18\x06 \x01(\tH\x01R\x12\x66\x65\x61turePackageName\x12\x32\n\x14\x66\x65\x61ture_service_name\x18\x07 \x01(\tH\x01R\x12\x66\x65\x61tureServiceName\x12>\n\x1cstate_update_entry_commit_id\x18\x08 \x01(\tR\x18stateUpdateEntryCommitId\x12\x31\n\x15join_key_column_names\x18\t \x03(\tR\x12joinKeyColumnNames\x12\x32\n\x15timestamp_column_name\x18\n \x01(\tR\x13timestampColumnName\x12\x38\n\x06schema\x18\x0b \x01(\x0b\x32 .tecton_proto.common.SparkSchemaR\x06schema\x12@\n\x04type\x18\x0c \x01(\x0e\x32,.tecton_proto.data.SavedFeatureDataFrameTypeR\x04typeB\x0f\n\rsource_fco_idB\x11\n\x0fsource_fco_name*?\n\x19SavedFeatureDataFrameType\x12\x0b\n\x07NOT_SET\x10\x00\x12\t\n\x05SAVED\x10\x01\x12\n\n\x06LOGGED\x10\x02\x42\x13\n\x0f\x63om.tecton.dataP\x01'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_id__pb2.DESCRIPTOR,tecton__proto_dot_common_dot_spark__schema__pb2.DESCRIPTOR,])

_SAVEDFEATUREDATAFRAMETYPE = _descriptor.EnumDescriptor(
  name='SavedFeatureDataFrameType',
  full_name='tecton_proto.data.SavedFeatureDataFrameType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NOT_SET', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SAVED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOGGED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1188,
  serialized_end=1251,
)
_sym_db.RegisterEnumDescriptor(_SAVEDFEATUREDATAFRAMETYPE)

SavedFeatureDataFrameType = enum_type_wrapper.EnumTypeWrapper(_SAVEDFEATUREDATAFRAMETYPE)
NOT_SET = 0
SAVED = 1
LOGGED = 2



_SAVEDFEATUREDATAFRAMEINFO = _descriptor.Descriptor(
  name='SavedFeatureDataFrameInfo',
  full_name='tecton_proto.data.SavedFeatureDataFrameInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tecton_proto.data.SavedFeatureDataFrameInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='name', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='tecton_proto.data.SavedFeatureDataFrameInfo.created_at', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='createdAt', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_archived', full_name='tecton_proto.data.SavedFeatureDataFrameInfo.is_archived', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='isArchived', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='workspace', full_name='tecton_proto.data.SavedFeatureDataFrameInfo.workspace', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='workspace', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_data_deleted', full_name='tecton_proto.data.SavedFeatureDataFrameInfo.is_data_deleted', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='isDataDeleted', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=175,
  serialized_end=384,
)


_SAVEDFEATUREDATAFRAME = _descriptor.Descriptor(
  name='SavedFeatureDataFrame',
  full_name='tecton_proto.data.SavedFeatureDataFrame',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='saved_feature_dataframe_id', full_name='tecton_proto.data.SavedFeatureDataFrame.saved_feature_dataframe_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='savedFeatureDataframeId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='info', full_name='tecton_proto.data.SavedFeatureDataFrame.info', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='info', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dataframe_location', full_name='tecton_proto.data.SavedFeatureDataFrame.dataframe_location', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='dataframeLocation', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_package_id', full_name='tecton_proto.data.SavedFeatureDataFrame.feature_package_id', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featurePackageId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_service_id', full_name='tecton_proto.data.SavedFeatureDataFrame.feature_service_id', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureServiceId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_package_name', full_name='tecton_proto.data.SavedFeatureDataFrame.feature_package_name', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featurePackageName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_service_name', full_name='tecton_proto.data.SavedFeatureDataFrame.feature_service_name', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='featureServiceName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='state_update_entry_commit_id', full_name='tecton_proto.data.SavedFeatureDataFrame.state_update_entry_commit_id', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='stateUpdateEntryCommitId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='join_key_column_names', full_name='tecton_proto.data.SavedFeatureDataFrame.join_key_column_names', index=8,
      number=9, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='joinKeyColumnNames', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp_column_name', full_name='tecton_proto.data.SavedFeatureDataFrame.timestamp_column_name', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestampColumnName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='schema', full_name='tecton_proto.data.SavedFeatureDataFrame.schema', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='schema', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='tecton_proto.data.SavedFeatureDataFrame.type', index=11,
      number=12, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='type', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='source_fco_id', full_name='tecton_proto.data.SavedFeatureDataFrame.source_fco_id',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='source_fco_name', full_name='tecton_proto.data.SavedFeatureDataFrame.source_fco_name',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=387,
  serialized_end=1186,
)

_SAVEDFEATUREDATAFRAMEINFO.fields_by_name['created_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SAVEDFEATUREDATAFRAME.fields_by_name['saved_feature_dataframe_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_SAVEDFEATUREDATAFRAME.fields_by_name['info'].message_type = _SAVEDFEATUREDATAFRAMEINFO
_SAVEDFEATUREDATAFRAME.fields_by_name['feature_package_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_SAVEDFEATUREDATAFRAME.fields_by_name['feature_service_id'].message_type = tecton__proto_dot_common_dot_id__pb2._ID
_SAVEDFEATUREDATAFRAME.fields_by_name['schema'].message_type = tecton__proto_dot_common_dot_spark__schema__pb2._SPARKSCHEMA
_SAVEDFEATUREDATAFRAME.fields_by_name['type'].enum_type = _SAVEDFEATUREDATAFRAMETYPE
_SAVEDFEATUREDATAFRAME.oneofs_by_name['source_fco_id'].fields.append(
  _SAVEDFEATUREDATAFRAME.fields_by_name['feature_package_id'])
_SAVEDFEATUREDATAFRAME.fields_by_name['feature_package_id'].containing_oneof = _SAVEDFEATUREDATAFRAME.oneofs_by_name['source_fco_id']
_SAVEDFEATUREDATAFRAME.oneofs_by_name['source_fco_id'].fields.append(
  _SAVEDFEATUREDATAFRAME.fields_by_name['feature_service_id'])
_SAVEDFEATUREDATAFRAME.fields_by_name['feature_service_id'].containing_oneof = _SAVEDFEATUREDATAFRAME.oneofs_by_name['source_fco_id']
_SAVEDFEATUREDATAFRAME.oneofs_by_name['source_fco_name'].fields.append(
  _SAVEDFEATUREDATAFRAME.fields_by_name['feature_package_name'])
_SAVEDFEATUREDATAFRAME.fields_by_name['feature_package_name'].containing_oneof = _SAVEDFEATUREDATAFRAME.oneofs_by_name['source_fco_name']
_SAVEDFEATUREDATAFRAME.oneofs_by_name['source_fco_name'].fields.append(
  _SAVEDFEATUREDATAFRAME.fields_by_name['feature_service_name'])
_SAVEDFEATUREDATAFRAME.fields_by_name['feature_service_name'].containing_oneof = _SAVEDFEATUREDATAFRAME.oneofs_by_name['source_fco_name']
DESCRIPTOR.message_types_by_name['SavedFeatureDataFrameInfo'] = _SAVEDFEATUREDATAFRAMEINFO
DESCRIPTOR.message_types_by_name['SavedFeatureDataFrame'] = _SAVEDFEATUREDATAFRAME
DESCRIPTOR.enum_types_by_name['SavedFeatureDataFrameType'] = _SAVEDFEATUREDATAFRAMETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SavedFeatureDataFrameInfo = _reflection.GeneratedProtocolMessageType('SavedFeatureDataFrameInfo', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDFEATUREDATAFRAMEINFO,
  '__module__' : 'tecton_proto.data.saved_feature_data_frame_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.SavedFeatureDataFrameInfo)
  })
_sym_db.RegisterMessage(SavedFeatureDataFrameInfo)

SavedFeatureDataFrame = _reflection.GeneratedProtocolMessageType('SavedFeatureDataFrame', (_message.Message,), {
  'DESCRIPTOR' : _SAVEDFEATUREDATAFRAME,
  '__module__' : 'tecton_proto.data.saved_feature_data_frame_pb2'
  # @@protoc_insertion_point(class_scope:tecton_proto.data.SavedFeatureDataFrame)
  })
_sym_db.RegisterMessage(SavedFeatureDataFrame)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
